#include "JEKGObj.h"
#include "JInteger.h"
#include "JIntegerData.h"
#include "JStringData.h"
#include "JIntegerProperty.h"
#include "JIntegerListProperty.h"
#include "JStringProperty.h"
#include "JLinkObj.h"
#include "JBytesData.h"


char* theJEKGObj = JEKGObj().Register();


const char* JEKGObj::className() const { return "JEKGObj";}


JObject* JEKGObj::clone() const { return new JEKGObj(*this);}


// class constructor
JEKGObj::JEKGObj() {
  igm = 1;
  ogm = 1 + 1 + 1 + 1 + 1 + 1 + 1 + 1 + 1;
  bytesProcessed = 0;
  out1 = 0;
  out2 = 0;
  out3 = 0;
  out4 = 0;
  out5 = 0;
  out6 = 0;
  out7 = 0;
  out8 = 0;
  ekgIdx = 0;
}


void JEKGObj::draw(JGraphics g, int x, int y, int w, int h) {
  JRect rect(x, y, w, h);
  drawText(g, JString("EKG"), rect);
}


boolean JEKGObj::inputAllowed(int n)
{ return !inputSet(n).last();}
JString JEKGObj::inputTag(int n) {
  char* tag[] = {
    "ekgStream" };
  return( tag[n] );
}


int JEKGObj::inputType( int n ) {
  return JBytesData::id;
}
JString JEKGObj::outputTag(int n) {
  char* tag[] = {
    "bytesProcessed",
    "out1",
    "out2",
    "out3",
    "out4",
    "out5",
    "out6",
    "out7",
    "out8" };
  return( tag[n] );
}


JFRect JEKGObj::inputArea(int n) {
  //if ( n < ( 1 )) {
    return leftArea( n, 0, 1, 0.1, 0.8 );
  //}
}


JFRect JEKGObj::outputArea(int n) {
  return rightArea(n, 0, 1 + 1 + 1 + 1 + 1 + 1 + 1 + 1 + 1);
}


int JEKGObj::inputFace(int n) {
  if ( n < 1 ) {
    return LEFT;
  } else {
    return TOP;
  }
}


int JEKGObj::outputFace(int n) {
  return RIGHT;
}


JFPoint JEKGObj::inputPoint(int n, JLinkObj& link) {
  return leftPoint( n, link, 0, 1, 0.1, 0.8);
}


JFPoint JEKGObj::outputPoint(int n, JLinkObj& link) {
  return rightPoint(n, link, 0, 1 + 1 + 1 + 1 + 1 + 1 + 1 + 1 + 1);
}


void JEKGObj::access(int n, JLinkObj& link, const JDataType& data) {
  if ( n == 0 ) {
    INT(data) = bytesProcessed;
  } else if ( n == 1 ) {
    INT(data) = out1;
  } else if ( n == 2 ) {
    INT(data) = out2;
  } else if ( n == 3 ) {
    INT(data) = out3;
  } else if ( n == 4 ) {
    INT(data) = out4;
  } else if ( n == 5 ) {
    INT(data) = out5;
  } else if ( n == 6 ) {
    INT(data) = out6;
  } else if ( n == 7 ) {
    INT(data) = out7;
  } else if ( n == 8 ) {
    INT(data) = out8;
  }
}


void JEKGObj::engine(int n, JLinkObj& link) {
  switch (n) {
  case EKGSTREAM: {
	  JBlock data;
	 link.access( JBlockData( data ));
	 int size = data.size();
	 data.Dereference();
	 for ( int i = 0; i < size; i++ ) {
		 processByte( (uchar)data[i] );
	 }
	 bytesProcessed += size;
	 broadcast( 0 );
				  }
    break;
  default:
    break;
  }
}

void JEKGObj::processByte( uchar b ) {
	shiftBytes();
	ekgBlock[ ekgIdx++ ] = b;
	checkBlock();
}

void JEKGObj::shiftBytes() {
	if ( ekgIdx >= 13 ) {
		for ( int i = 0; i < 12; i++ ) {
			ekgBlock[i] = ekgBlock[ i + 1 ];
		}
		ekgIdx = 12;
	}
}

void JEKGObj::checkBlock() {
	if ( ekgIdx == 13 ) {
		int sum = 0;
		for ( int i = 0; i < 12; i++ ) {
			sum += ekgBlock[i];
		}
		if (( sum & 0xff ) == ekgBlock[12] ) {
			blastOutput();
			ekgIdx = 0;
		}
	}
}

void JEKGObj::blastOutput() {
	out1 = ( ekgBlock[0] << 4 ) + ( ekgBlock[1] >> 4 );
	broadcast( 1 );
	out2 = (( ekgBlock[1] & 0xf ) << 8 ) + ekgBlock[2];
	broadcast( 2 );
	out3 = ( ekgBlock[3] << 4 ) + ( ekgBlock[4] >> 4 );
	broadcast( 3 );
	out4 = (( ekgBlock[4] & 0xf ) << 8 ) + ekgBlock[5];
	broadcast( 4 );
	out5 = ( ekgBlock[6] << 4 ) + ( ekgBlock[7] >> 4 );
	broadcast( 5 );
	out6 = (( ekgBlock[7] & 0xf ) << 8 ) + ekgBlock[8];
	broadcast( 6 );
	out7 = ( ekgBlock[9] << 4 ) + ( ekgBlock[10] >> 4 );
	broadcast( 7 );
	out8 = (( ekgBlock[10] & 0xf ) << 8 ) + ekgBlock[11];
	broadcast( 8 );
}


